import {Component, OnInit, ViewChild} from '@angular/core';
import {
  DayService,
  WeekService,
  WorkWeekService,
  MonthService,
  AgendaService,
  ResizeService,
  DragAndDropService,
  EventSettingsModel,
  ScheduleComponent,
  View,
  EventRenderedArgs,
  PopupOpenEventArgs,
  TimelineYearService,CellClickEventArgs,ActionEventArgs, EJ2Instance
} from '@syncfusion/ej2-angular-schedule';
import {extend} from '@syncfusion/ej2-base';
import {scheduleData} from './data';
import {loadCldr} from '@syncfusion/ej2-base';
import {L10n} from '@syncfusion/ej2-base';
import {Subject, Subscription} from "rxjs";
import {debounceTime, distinctUntilChanged, map} from "rxjs/operators";
import {ElasticsearchService} from "app/shared/services/elasticsearch.service";
import {Betrieb} from "app/shared/models/betrieb.model";
import {GescheftbDTO, GsbadresseDTOList, GsbkomunikationDTOList} from "app/shared/models/gescheftbDTO.model";
import {Actioncode} from "app/shared/enum/enum.model";
import {Bp} from "app/shared/models/bp.model";
import {Liland} from "app/shared/models/liland.model";
import {sortSuggestion} from "app/shared/util/sort-util";
import {rechform} from "app/shared/models/rechtform.model";
import {AccountService} from "app/core/auth/account.service";
import {BpService} from "app/shared/services/bp.service";
import {RechtformService} from "app/shared/services/rechtform.service";
import {LilandService_bp} from "app/shared/services/liland-bp.service";
import {DashboardModel} from "app/shared/models/dashboard.model";
import {Listerform} from "app/shared/models/listerform.model";
import {BpBet} from "app/shared/models/bpBet.model";
import {BpBetService} from "app/shared/services/bpBet.service";
import {BetriebService} from "app/shared/services/betrieb.service";
import {ToastService} from "app/shared/services/toast.service";
import { Dialog } from '@syncfusion/ej2-popups';


var numberingSystems = require('cldr-data/supplemental/numberingSystems.json');
var gregorian = require('cldr-data/main/de/ca-gregorian.json');
var numbers = require('cldr-data/main/de/numbers.json');
var timeZoneNames = require('cldr-data/main/de/timeZoneNames.json');

loadCldr(numberingSystems, gregorian, numbers, timeZoneNames);

L10n.load({
  'de': {
    'schedule': {
      'day': 'tag',
      'week': 'woche',
      'workWeek': 'arbitewoche',
      'month': 'monat',
      'today': 'heute',
      "newEvent": "New Event",
    }
  }
});

@Component({
  selector: 'app-calendar',
  templateUrl: './calendar.component.html',
  providers: [DayService, WeekService, WorkWeekService, MonthService, AgendaService, ResizeService, DragAndDropService, TimelineYearService],
  styleUrls: [
    './calendar.component.scss'
  ]
})

export class CalendarComponent implements OnInit {

  public selectedDate: Date = new Date(2019, 0, 10);
  public eventSettings: EventSettingsModel = {dataSource: <Object[]>extend([], scheduleData, null, true)};
  public isSelected: Boolean = true;
  public dayInterval: number = 3;
  public weekInterval: number = 2;
  public monthInterval: number = 2;
  public spielzeit: number = 12;
  @ViewChild('scheduleObj', {static: false}) public scheduleObj: ScheduleComponent;
  public scheduleView: View = 'Week';
  public viewInterval: number = 6;
  public datas: string[] = ['Day', 'Week', 'WorkWeek', 'Month'];
  public currentView: View = 'Month';

  private subscription: Subscription;
  public keyUp = new Subject<any>();
  public queryStringForElasticSearch: Array<any> = [];
  public resultFound: Array<any> = [];
  public searchString: string = null;
  public resultLength = null;
  public showResultList: boolean = false;
  public productionResultNotFound: boolean = false;
  public productionVetragespartner = null;
  public productionSelected: boolean = false;
  public productionVetrageCreate = null;
  public prodVetrageGotIt: boolean = false;

  companyObjectForProd: Betrieb = new Betrieb(
    null,
    null,
    null,
    [],
    null,
    [],
    [],
    null,
    null,
    0,
    0,
    null,
    null,
    null,
    null,
    0,
    0,
    0,
    0,
    2,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    false,
    null,
    true,
    null,
    null,
    null,
    null,
    null,
    null,
    [],
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    null,
    0,
    null
  );
  testgescheftb = new GescheftbDTO(
    null,
    null,
    null,
    null,
    null,
    null,
    0,
    [new GsbadresseDTOList(Actioncode.unchanged)],
    [new GsbkomunikationDTOList(Actioncode.unchanged)],
    null,
    null,
    null,
    null,
    null,
    null,
    0
  );
  public searchFailedBetForProd: boolean = false;
  account: DashboardModel;
  bp: Bp = new Bp();
  liland;
  listerform: Listerform[] = [];
  aktivBetrieb: BpBet = null;
  aktivBetriebObject: Betrieb = null;
  modalshoworhide: boolean = false;
  public currentEvent;

  constructor(private es: ElasticsearchService,
              private betriebService: BetriebService,
              private principal: AccountService,
              private bpService: BpService,
              private listerformService: RechtformService,
              private landService: LilandService_bp,
              private aktiv_betriebService: BpBetService,
              public toastService: ToastService,
  ) {
    this.subscription = this.keyUp
      .pipe(
        map(event => event.target.value),
        debounceTime(700),
        distinctUntilChanged()
      )
      .subscribe(x => this.searchWithInputField(x));
  }

  ngOnInit() {
    this.principal.identity().subscribe(account => {
      this.account = account;
      this.bpService.find().subscribe((bp: Bp) => {
        this.bp = bp;
        this.loadAktivBetrieb(bp.id);
      });
    });

    this.landService.query().subscribe((land: Liland[]) => {
      this.liland = sortSuggestion(land);
    });

    this.listerformService.query().subscribe((rechtforms: rechform[]) => {
      this.listerform = sortSuggestion(rechtforms);
    });
  }

  loadAktivBetrieb(id) {
    this.aktiv_betriebService
      .query({
        bpsId: id
      })
      .subscribe((response: BpBet[]) => {
        if (response.length != 0) {
          this.aktivBetrieb = response[0];
          this.testgescheftb.betriebsId = this.aktivBetrieb.betriebsId;
          this.betriebService.findCrowd(this.aktivBetrieb.betriebsId).subscribe((betrieb: Betrieb) => {
            this.aktivBetriebObject = betrieb;
          });
        }
      });
  }

  search(queryString) {
    this.productionResultNotFound = false;
    this.productionVetragespartner = null;
    this.productionSelected = false;
    this.es.fullTextSearchFilter('prod', '', '', queryString).subscribe(
      response => {
        let res: any = response;
        this.resultLength = res.hits.total;
        if (res.hits.hits.length != 0) {
          res.hits.hits.forEach(item => this.resultFound.push(item));
          this.showResultList = true;
        } else {
          this.productionResultNotFound = true;
          console.log('Not Found at elastic: ');
        }
      },
      error => {
        console.error(error);
      }
    );
  }

  searchWithInputField(searchString) {
    let stringList: Array<string> = searchString
      .replace(/["&/(){}[]/gi, '')
      .trim()
      .split(' ');
    if (stringList.length != 0) {
      this.queryStringForElasticSearch = this.queryStringForElasticSearch.filter(item => !item.hasOwnProperty('query_string'));

      for (let string of stringList) {
        if (string != '') {
          let keyWildcard = string + '*';
          let matchobjTemp = {query_string: {}};
          matchobjTemp.query_string['query'] = keyWildcard;
          matchobjTemp.query_string['fields'] = [
            'betriebsname',
            'prodWerkSearchTeilDTO.titel',
            'beschreibung',
            'titel',
            'vorname',
            'nachname',
            'interne_bezeichnung_der_produkts'
          ];
          this.queryStringForElasticSearch.push(matchobjTemp);
        }
      }

      if (this.queryStringForElasticSearch.length != 0) {
        this.resultFound = [];
        this.search(this.queryStringForElasticSearch);
      } else {
        this.resultFound = [];
      }
    }
  }

  clearSearchVal(event) {
    console.log("selected Production: ", event)
    this.productionSelected = true;
    this.searchString = event.prodWerkSearchTeilDTO.titel;
    this.showResultList = false;
    this.productionVetragespartner = event.angeProdSearchDTO.betriebsname;
  }

  selectedItemBetrieb(something: any) {
    //elastic code
    console.log("Bet selected: ", something)
    this.companyObjectForProd.betriebsname = something.betriebsname;
    this.productionVetragespartner = something.betriebsname;
    this.prodVetrageGotIt = true;
  }

  hideResultBox(val) {
    this.prodVetrageGotIt = false;
    this.searchFailedBetForProd = false;
    this.companyObjectForProd.betriebsname = null;
  }

  resultnotFound(event) {
    console.log('not found at prod string: ', event.searchString);
    this.searchFailedBetForProd = true;
    this.companyObjectForProd.betriebsname = event.searchString;
    this.productionVetragespartner = event.searchString;
  }

  talkBack(e?: any) {

    this.productionVetrageCreate = e;
    this.prodVetrageGotIt = true;
    this.showSuccess({
      title: 'Einladung gesendet',
      msg: 'Eine Einladungs-E-Mail wurde an die angegebene Adresse gesendet.',
      timeout: 5000
    });
  }

  closeModalForAngeAfterCreate(i: boolean) {
    console.log("hide modal: ", i)
    if (i) {
      this.modalshoworhide = true;
    } else {
      this.modalshoworhide = false;
    }
  }

  showSuccess(options: any) {
    this.toastService.show(options.msg, {
      classname: 'bg-success',
      delay: options.timeout,
      autohide: true,
      headertext: options.title
    });
  }
  oneventRendered(args: EventRenderedArgs): void {
    const eventColor: string = args.data.EventColor as string;
    if (!args.element || !eventColor) {
      return;
    } else {
      args.element.style.backgroundColor = eventColor;
    }
  }

  public eventAdd(e) {
    const data: Object = this.scheduleObj.getCellDetails(this.scheduleObj.getSelectedElements()) as Object;
    const eventData: { [key: string]: Object } = this.scheduleObj.eventWindow.getObjectFromFormData('e-schedule-dialog');
    this.scheduleObj.eventWindow.convertToEventData(data as { [key: string]: Object }, eventData);
    eventData.Id = this.scheduleObj.eventBase.getEventMaxID() as number + 1;
    this.scheduleObj.addEvent(eventData);
    this.dialogClose();
  }
  public eventDelete(e) {
    const eventData: { [key: string]: Object } = this.scheduleObj.activeEventData.event as any;
    this.scheduleObj.deleteEvent(eventData);
    this.dialogClose();
  }
  public editEvent(e) {
    debugger;
    const eventData: { [key: string]: Object } = this.scheduleObj.eventWindow.getObjectFromFormData('e-schedule-dialog');
    eventData.Id = this.currentEvent.Id;
    this.scheduleObj.saveEvent(eventData);
    this.dialogClose();
  }
  public dialogClose() {
    let dialogObj: Dialog = (document.querySelector('.e-schedule-dialog') as EJ2Instance).ej2_instances[0] as Dialog;
    dialogObj.hide();
  }

  public onPopupOpen(args: PopupOpenEventArgs): void {
        if(args.type == 'QuickInfo') {
          let cellData: CellClickEventArgs = this.scheduleObj.getCellDetails(args.target);
          let eventData: { [key: string]: Object } = {
            StartTime: cellData.startTime,
            EndTime: cellData.endTime,
            IsAllDay: cellData.isAllDay,
          };
         let slides =  document.getElementsByClassName('e-cell-popup');
          for (let i = 0; i < slides.length; i++) {
            const slide = slides[i] as HTMLElement;
            slide.style.display = "none";
          }
          this.scheduleObj.openEditor(eventData, 'Add', true);
        } else if(args.type == 'Editor'){
        }
  }


  onActionBegin(args: ActionEventArgs): void {
    if (args.requestType == 'eventCreate' || args.requestType == 'eventChange') {
      // args.data
      console.log("Args data: ", args.data)
    let data = {
        Id: 1,
        Subject: "New Event",
        StartTime: new Date(2014, 4, 5, 10, 0),
        EndTime: new Date(2014, 4, 5, 12, 0)};
      if (args.requestType === 'eventChange') { //while editing the existing event
        console.log("event edit");
      }
      if (args.requestType === 'eventCreate') { //while creating new event
        console.log("event create");
        args.cancel = true;
        this.scheduleObj.addEvent(data);

      }
    };
  }

  public appWindow(args: any): void {
    // (cellClick)= "appWindow($event)"
    console.log("Popup hide class :", args)
    console.log("schedular Object :", this.scheduleObj)
    args.cancel = true;
    let eventData: { [key: string]: Object } = {
      StartTime: args.startTime,
      EndTime: args.endTime,
      IsAllDay: args.isAllDay,
    };
    /*let slides =  document.getElementsByClassName('e-cell-popup');
    for (let i = 0; i < slides.length; i++) {
      const slide = slides[i] as HTMLElement;
      slide.style.display = "none";
    }*/
    this.scheduleObj.openEditor(eventData, 'Add', true);  }

}
